#include "qArray.lsl"
#include "qLab/qPosition.lsl"
#include "qLab/qDynaPrim/qDynaPrim.lsl"

float hedgeSize = 3;
integer mazeWidth = 13;

qArray maze;
vector mazeCorner;

list prims;

vector red = <1, 0, 0>;
vector green = <0, 1, 0>;

vector array2World(vector arrayCoord)
{
  return (mazeCorner + hedgeSize * arrayCoord);
}


hedgeMake(integer prim, vector arrayCoord)
{
  vector position = array2World(arrayCoord);

  llShout(prim, "prim -setObjectName maze");
  llShout(prim,
	  "texture -setTexture 862df87c-fe09-1778-1a10-5fb2b03dec66 -1");
  llShout(prim, 
	  "transform" +
	  " -setScale " + ((string)hedgeSize + " " +
			   (string)hedgeSize + " "  +
			   (string)hedgeSize + " ") +
	  " -gotoGlobalPos " + ((string)position.x + " " +
			       (string)position.y + " " +
			       (string)position.z + " "));

}



mazeGenerate()
{
  llSay(0, "generating maze.");

  llSetColor(red, ALL_SIDES);

  maze = qArrayCreate([mazeWidth, mazeWidth]);

  mazeCorner = qGetGlobalPosition() + (<hedgeSize/2 + 0.4, 
				       -(mazeWidth/2 + 1) * hedgeSize, 
				       0>);
  integer x;
  integer y;
  for (x = 0; x < mazeWidth; x++)
    {
      llSetColor(red * (x % 2), ALL_SIDES);
      for (y = 0; y < mazeWidth; y++)
	{
	  integer hedgeHere;
	  if ((x == 0) ||
	      (x == mazeWidth - 1) ||
	      (y == 0) ||
	      (y == mazeWidth - 1) ||
	      ((x % 2 == 0) && (y % 2 == 0)) ||
	      ((llFrand(1.0) < 0.5) && (((x + y) % 2) == 1)))
	    {
	      hedgeHere = 1;
	    }
	  else
	    hedgeHere = 0;

	  maze = qArraySetInteger(maze, [x, y], hedgeHere);
	}
    }

  maze = qArraySetInteger(maze, [0, (integer) mazeWidth/2], 0);
  maze = qArraySetInteger(maze, [1, (integer) mazeWidth/2], 0);



  for (x = 0; x < mazeWidth; x++)
    {
      llSetColor(red * (x % 2), ALL_SIDES);
      for (y = 0; y < mazeWidth; y++)
	{
	  if (qArrayGetInteger(maze, [x, y]))
	    {
	      integer prim = qDynaRez();
	      hedgeMake(prim, <x, y, 0>);
	      maze = qArraySetInteger(maze, [x,y], prim);
	    }

	}
    }
  
  qDynaCleanup();

  llSetColor(green, ALL_SIDES);
}



mazeDelete()
{
  llSay(0, "deleting maze.");

  integer x;
  integer y;

  for (x = 0; x < mazeWidth; x++)
    {
      llSetColor(red * (x % 2), ALL_SIDES);
      for (y = 0; y < mazeWidth; y++)
	{
	  integer prim = qArrayGetInteger(maze, [x, y]);

	  if (prim != 0)
	    llShout(prim, "prim -die");
	}
    }
}


mazeMove(integer prim, vector original, vector destination)
{
  vector position = array2World(destination);
  llShout(prim, "transform -gotoGlobalPos " + ((string)position.x + " " +
					       (string)position.y + " " +
					       (string)position.z + " "));

  maze = qArraySetInteger(maze, [original.x, original.y], 0);
  maze = qArraySetInteger(maze, [destination.x, destination.y], prim);
}



mazeMoveWall()
{
  integer i;

  for(i = 0; i < 20; i++)  // no infinite loops, please
    {
      integer origX = (integer)llFrand(mazeWidth - 2) + 1;
      integer origY = (integer)llFrand(mazeWidth - 2) + 1;
      vector orig = <origX, origY, 0>;

      if (((integer)(orig.x + orig.y) % 2) == 1) // is movable wall?
	{
	  integer prim = qArrayGetInteger(maze, [orig.x, orig.y]);
	  if (prim) // is there?
	    {
	      list deltas = [ <-1, -1, 0>, <-1, 1, 0>, <1, -1, 0>, <1, 1, 0> ];
	      deltas = llListRandomize(deltas, 1);

	      integer j;
	      for (j = 0; j < llGetListLength(deltas); j++)
		{
		  vector delta = llList2Vector(deltas, j);
		  vector new = orig + delta;
		  
		  // don't block maze entrance
		  if ((new.x != 1) || (new.y != (integer)(mazeWidth/2))) 
		    {
		      if (!qArrayGetInteger(maze, [new.x, new.y])) // is empty?
			{

			  mazeMove(prim, orig, new);
			  return;
			}
		    }
		}
	    }
	}
    }
}


default
{
  state_entry()
    {
      llSetTimerEvent(5);
      mazeGenerate();
    }

  touch(integer n)
    {
      mazeDelete();
      mazeGenerate();
    }

  timer()
    {
      mazeMoveWall();
    }
}
